@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.216 2006/01/19 11:21:33 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.220 2006/01/29 13:44:02 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -452,6 +452,9 @@ bacon_video_widget_finalize (GObject *object)
 {
 	BaconVideoWidget *bvw = (BaconVideoWidget *) object;
 
+	if (bvw->priv->gc)
+		g_object_unref (G_OBJECT (bvw->priv->gc));
+
 	if (bvw->priv->xine != NULL) {
 		xine_plugins_garbage_collector (bvw->priv->xine);
 		xine_exit (bvw->priv->xine);
@@ -1423,6 +1426,23 @@ xine_event (void *user_data, const xine_event_t *event)
 	}
 }
 
+#if GLIB_CHECK_VERSION(2,8,0)
+static int
+bacon_video_widget_sort_queue (gconstpointer a, gconstpointer b, gpointer data)
+{
+	signal_data *one, *two;
+
+	one = (signal_data *) a;
+	two = (signal_data *) b;
+
+	if (one->signal == two->signal)
+		return 0;
+	if (one->signal == ERROR_ASYNC || one->signal == MESSAGE_ASYNC)
+		return -1;
+	return 1;
+}
+#endif /* GLIB_CHECK_VERSION 2.8.0 */
+
 static void
 xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 {
@@ -1433,12 +1453,17 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 
 	sched_yield ();
 
+#if GLIB_CHECK_VERSION(2,8,0)
+	/* Sort the queue with the errors first */
+	g_async_queue_sort (bvw->priv->queue, bacon_video_widget_sort_queue, bvw);
+#endif
+
 	/* Steal messages from the async queue, if there's an error,
 	 * to use as the error message rather than the crappy errors from
 	 * xine_open() */
 	while ((data = g_async_queue_try_pop (bvw->priv->queue)) != NULL)
 	{
-		if (data->signal == ERROR_ASYNC)
+		if (data->signal == ERROR_ASYNC || data->signal == MESSAGE_ASYNC)
 		{
 			if (save_data != NULL)
 			{
@@ -1447,8 +1472,8 @@ xine_try_error (BaconVideoWidget *bvw, gboolean probe_error, GError **error)
 			}
 			save_data = data;
 		} else {
-			g_free (data->msg);
-			g_free (data);
+			g_async_queue_push (bvw->priv->queue, data);
+			break;
 		}
 	}
 
@@ -3565,7 +3590,7 @@ GList
 	num_channels = xine_get_stream_info
 		(bvw->priv->stream, XINE_STREAM_INFO_MAX_SPU_CHANNEL);
 
-	if (num_channels < 2)
+	if (num_channels < 1)
 		return NULL;
 
 	for(i = 0; i < num_channels; i++)
