@@ -1105,12 +1105,23 @@ parse_asx_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 		if (node->name == NULL)
 			continue;
 
-		/* ENTRY should only have one ref and one title nodes */
+		/* ENTRY can only have one title node but multiple REFs */
 		if (g_ascii_strcasecmp ((char *)node->name, "ref") == 0
 				|| g_ascii_strcasecmp ((char *)node->name, "entryref") == 0) {
-			url = xmlGetProp (node, (guchar *)"href");
-			if (url == NULL)
-				url = xmlGetProp (node, (guchar *)"HREF");
+			unsigned char *tmp;
+
+			tmp = xmlGetProp (node, (guchar *)"href");
+			if (tmp == NULL)
+				tmp = xmlGetProp (node, (guchar *)"HREF");
+			if (tmp == NULL)
+				continue;
+			if (url == NULL || g_str_has_prefix ((char *)tmp, "mms:") != FALSE) {
+				g_free (url);
+				url = tmp;
+			} else {
+				g_free (tmp);
+			}
+
 			continue;
 		}
 
@@ -1274,7 +1285,7 @@ parse_smil_entry (TotemPlParser *parser, char *base, xmlDocPtr doc,
 			continue;
 
 		/* ENTRY should only have one ref and one title nodes */
-		if (g_ascii_strcasecmp ((char *)node->name, "video") == 0) {
+		if (g_ascii_strcasecmp ((char *)node->name, "video") == 0 || g_ascii_strcasecmp ((char *)node->name, "audio") == 0) {
 			url = xmlGetProp (node, (guchar *)"src");
 			title = xmlGetProp (node, (guchar *)"title");
 
@@ -1535,6 +1546,7 @@ static PlaylistTypes special_types[] = {
 	{ "audio/x-ms-asx", totem_pl_parser_add_asx },
 	{ "audio/x-scpls", totem_pl_parser_add_pls },
 	{ "application/x-smil", totem_pl_parser_add_smil },
+	{ "application/smil", totem_pl_parser_add_smil },
 	{ "application/x-gnome-app-info", totem_pl_parser_add_desktop },
 	{ "application/x-desktop", totem_pl_parser_add_desktop },
 	{ "x-directory/normal", totem_pl_parser_add_directory },
@@ -1640,10 +1652,18 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 	gpointer data = NULL;
 	TotemPlParserResult ret = FALSE;
 	char *super;
+	gboolean found;
 
 	if (parser->priv->recurse_level > RECURSE_LEVEL_MAX)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
+	/* Shouldn't gnome-vfs have a list of schemes it supports? */
+	if (g_str_has_prefix (url, "mms") != FALSE
+			|| g_str_has_prefix (url, "rtsp") != FALSE) {
+		totem_pl_parser_add_one_url (parser, url, NULL);
+		return TOTEM_PL_PARSER_RESULT_SUCCESS;
+	}
+
 	mimetype = gnome_vfs_mime_type_from_name (url);
 	if (mimetype == NULL || strcmp (GNOME_VFS_MIME_TYPE_UNKNOWN, mimetype) == 0) {
 		mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data);
@@ -1657,11 +1677,35 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 		return TOTEM_PL_PARSER_RESULT_IGNORED;
 	}
 
+	parser->priv->recurse_level++;
+
+	found = FALSE;
+	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
+		if (strcmp (special_types[i].mimetype, mimetype) == 0) {
+			ret = (* special_types[i].func) (parser, url, data);
+			found = TRUE;
+			break;
+		}
+	}
+
+	for (i = 0; i < G_N_ELEMENTS(dual_types) && found == FALSE; i++) {
+		if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
+			if (data == NULL) {
+				mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data);
+			}
+			ret = (* dual_types[i].func) (parser, url, data);
+			break;
+		}
+	}
+
+	g_free (data);
+
+	parser->priv->recurse_level--;
+
 	super = gnome_vfs_get_supertype_from_mime_type (mimetype);
 	for (i = 0; i < G_N_ELEMENTS (ignore_types); i++) {
 		if (gnome_vfs_mime_type_is_supertype (ignore_types[i].mimetype) != FALSE) {
 			if (strcmp (super, ignore_types[i].mimetype) == 0) {
-				g_free (data);
 				g_free (super);
 				return TOTEM_PL_PARSER_RESULT_IGNORED;
 			}
@@ -1670,34 +1714,12 @@ totem_pl_parser_parse_internal (TotemPlParser *parser, const char *url)
 
 			eq = gnome_vfs_mime_type_get_equivalence (mimetype, ignore_types[i].mimetype);
 			if (eq == GNOME_VFS_MIME_PARENT || eq == GNOME_VFS_MIME_IDENTICAL) {
-				g_free (data);
+				g_free (super);
 				return TOTEM_PL_PARSER_RESULT_IGNORED;
 			}
 		}
 	}
-
-	parser->priv->recurse_level++;
-
-	for (i = 0; i < G_N_ELEMENTS(special_types); i++) {
-		if (strcmp (special_types[i].mimetype, mimetype) == 0) {
-			ret = (* special_types[i].func) (parser, url, data);
-			g_free (data);
-			break;
-		}
-	}
-
-	for (i = 0; i < G_N_ELEMENTS(dual_types); i++) {
-		if (strcmp (dual_types[i].mimetype, mimetype) == 0) {
-			if (data == NULL) {
-				mimetype = my_gnome_vfs_get_mime_type_with_data (url, &data);
-			}
-			ret = (* dual_types[i].func) (parser, url, data);
-			g_free (data);
-			break;
-		}
-	}
-
-	parser->priv->recurse_level--;
+	g_free (super);
 
 	if (ret != TOTEM_PL_PARSER_RESULT_SUCCESS && parser->priv->fallback) {
 		totem_pl_parser_add_one_url (parser, url, NULL);
