@@ -26,6 +26,7 @@
 
 #include <gdk/gdkx.h>
 #include <glade/glade.h>
+#include <gconf/gconf-client.h>
 
 #include <dbus/dbus-glib.h>
 #include <sys/types.h>
@@ -42,6 +43,11 @@ GtkWidget *totem_volume_create (void);
 
 #define OPTION_IS(x) (strcmp(argv[i], x) == 0)
 
+/* For newer D-Bus version */
+#ifndef DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT
+#define DBUS_NAME_FLAG_PROHIBIT_REPLACEMENT 0
+#endif
+
 typedef enum {
 	STATE_PLAYING,
 	STATE_PAUSED,
@@ -71,6 +77,7 @@ typedef struct _TotemEmbedded {
 	/* XEmbed */
 	gboolean embedded_done;
 } TotemEmbedded;
+
 G_DEFINE_TYPE (TotemEmbedded, totem_embedded, G_TYPE_OBJECT);
 static void totem_embedded_class_init (TotemEmbeddedClass *klass) { }
 static void totem_embedded_init (TotemEmbedded *emb) { }
@@ -161,10 +168,11 @@ totem_embedded_open (TotemEmbedded *emb)
 		//disp = gnome_vfs_unescape_string_for_display (totem->mrl);
 		msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 		g_free (disp);
-		if (err != NULL)
+		if (err != NULL) {
 			g_message ("error: %s", err->message);
-		totem_interface_error_blocking (msg, err->message,
+			totem_interface_error_blocking (msg, err->message,
 				GTK_WINDOW (emb->window));
+		}
 
 		g_free (msg);
 
@@ -201,6 +209,37 @@ totem_embedded_stop (TotemEmbedded *emb, GError **err)
 	return TRUE;
 }
 
+static GdkWindow *
+totem_gtk_plug_get_toplevel (GtkPlug *plug)
+{
+	Window root, parent, *children;
+	guint nchildren;
+	GdkNativeWindow xid;
+
+	g_return_val_if_fail (GTK_IS_PLUG (plug), NULL);
+
+	xid = gtk_plug_get_id (plug);
+
+	do
+	{
+		if (XQueryTree (GDK_DISPLAY (), xid, &root,
+					&parent, &children, &nchildren) == 0)
+		{
+			g_warning ("Couldn't find window manager window");
+			return None;
+		}
+
+		if (root == parent) {
+			GdkWindow *toplevel;
+			toplevel = gdk_window_foreign_new (xid);
+			return toplevel;
+		}
+
+		xid = parent;
+	}
+	while (TRUE);
+}
+
 static void
 on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 {
@@ -242,17 +281,30 @@ on_about1_activate (GtkButton *button, TotemEmbedded *emb)
 	g_free (backend_version);
 	g_free (description);
 
+	if (GTK_IS_PLUG (emb->window)) {
+		GdkWindow *toplevel;
+
+		gtk_widget_realize (emb->about);
+		toplevel = totem_gtk_plug_get_toplevel (GTK_PLUG (emb->window));
+		if (toplevel != NULL) {
+			gdk_window_set_transient_for
+				(GTK_WIDGET (emb->about)->window, toplevel);
+			gdk_window_unref (toplevel);
+		}
+	} else {
+		gtk_window_set_transient_for (GTK_WINDOW (emb->about),
+				GTK_WINDOW (emb->window));
+	}
+
 	if (pixbuf != NULL)
 		gdk_pixbuf_unref (pixbuf);
 
 	g_object_add_weak_pointer (G_OBJECT (emb->about),
 			(gpointer *)&emb->about);
-	gtk_window_set_transient_for (GTK_WINDOW (emb->about),
-			GTK_WINDOW (emb->window));
 	g_signal_connect (G_OBJECT (emb->about), "response",
 			G_CALLBACK (gtk_widget_destroy), NULL);
 
-	gtk_widget_show(emb->about);
+	gtk_widget_show (emb->about);
 }
 
 static void
@@ -369,6 +421,8 @@ totem_embedded_add_children (TotemEmbedded *emb)
 {
 	GtkWidget *child, *container, *pp_button, *vbut;
 	GError *err = NULL;
+	GConfClient *gc;
+	int volume;
 
 	emb->xml = totem_interface_load_with_root ("mozilla-viewer.glade",
 			"vbox1", _("Plugin"), TRUE,
@@ -416,9 +470,13 @@ totem_embedded_add_children (TotemEmbedded *emb)
 	g_signal_connect (G_OBJECT (pp_button), "clicked",
 			  G_CALLBACK (on_play_pause), emb);
 
+	gc = gconf_client_get_default ();
+	volume = gconf_client_get_int (gc, GCONF_PREFIX"/volume", NULL);
+	g_object_unref (G_OBJECT (gc));
+
+	bacon_video_widget_set_volume (emb->bvw, volume);
 	vbut = glade_xml_get_widget (emb->xml, "volume_button");
-	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut),
-			bacon_video_widget_get_volume (emb->bvw));
+	bacon_volume_button_set_value (BACON_VOLUME_BUTTON (vbut), volume);
 	g_signal_connect (G_OBJECT (vbut), "value-changed",
 			  G_CALLBACK (cb_vol), emb);
 
@@ -475,9 +533,9 @@ int main (int argc, char **argv)
 	TotemEmbedded *emb;
 	DBusGProxy *proxy;
 	DBusGConnection *conn;
+	Window xid;
 	int i;
 	guint res;
-	Window xid;
 	gchar *svcname;
 	GError *e = NULL;
 
@@ -489,7 +547,6 @@ int main (int argc, char **argv)
 
 	g_thread_init (NULL);
 	bacon_video_widget_init_backend (NULL, NULL);
-	//gdk_threads_init ();
 
 	gtk_init (&argc, &argv);
 	dbus_g_object_type_install_info (TOTEM_TYPE_EMBEDDED,
@@ -565,12 +622,14 @@ int main (int argc, char **argv)
 
 		/* The miraculous XEMBED protocol */
 		window = gtk_plug_new ((GdkNativeWindow)xid);
-		gtk_signal_connect(GTK_OBJECT(window), "embedded",
+		gtk_signal_connect (GTK_OBJECT(window), "embedded",
 				G_CALLBACK (embedded), NULL);
 		gtk_widget_realize (window);
 
 		emb->window = window;
 	} else {
+		if (emb->filename == NULL)
+			return 1;
 		/* Stand-alone version */
 		emb->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
 	}
