@@ -194,6 +194,7 @@ totem_action_exit (Totem *totem)
 	totem_action_fullscreen (totem, FALSE);
 
 	totem_sublang_exit (totem);
+	totem_destroy_file_filters ();
 	totem_named_icons_dispose (totem);
 
 	if (totem->bvw)
@@ -202,6 +203,9 @@ totem_action_exit (Totem *totem)
 				bacon_video_widget_get_volume (totem->bvw),
 				NULL);
 
+	if (totem->gc)
+		g_object_unref (G_OBJECT (totem->gc));
+
 	if (totem->playlist)
 		gtk_widget_destroy (GTK_WIDGET (totem->playlist));
 	if (totem->win)
@@ -798,13 +802,20 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 			disp = totem_uri_escape_for_display (totem->mrl);
 			msg = g_strdup_printf(_("Totem could not play '%s'."), disp);
 			g_free (disp);
-			totem_action_error (msg, err->message, totem);
+			if (err && err->message) {
+				totem_action_error (msg, err->message, totem);
+			}
+			else {
+				totem_action_error (msg, _("No error message"), totem);
+			}
 			g_free (msg);
 		}
 
 		if (retval == FALSE)
 		{
-			g_error_free (err);
+			if (err) {
+				g_error_free (err);
+			}
 			g_free (totem->mrl);
 			totem->mrl = NULL;
 			play_pause_set_label (totem, STATE_STOPPED);
@@ -1387,8 +1398,12 @@ seek_slider_pressed_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 
 	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
 	{
-		totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-		bacon_video_widget_pause (totem->bvw);
+		if (!totem->seek_in_progress) {
+			totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
+			bacon_video_widget_pause (totem->bvw);
+			/* We need to remember that we are already paused and seeking */
+			totem->seek_in_progress = TRUE;
+		}
 	}
 
 	return FALSE;
@@ -1433,6 +1448,7 @@ seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	{
 		if (totem->was_playing)
 			bacon_video_widget_play (totem->bvw, NULL);
+		totem->seek_in_progress = FALSE;
 	} else {
 		totem_action_seek (totem, val / 65535.0);
 	}
@@ -2761,6 +2777,8 @@ on_window_key_press_event (GtkWidget *win, GdkEventKey *event, Totem *totem)
 		case GDK_o:
 		case GDK_L:
 		case GDK_l:
+		case GDK_S:
+		case GDK_s:
 		case GDK_Right:
 		case GDK_Left:
 			return totem_action_handle_key (totem, event);
